/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.util.scan.api;

import org.ow2.util.archive.api.IArchive;

/**
 * ArchiveScanner Factory is the class you should use to construct
 * ArchiveScanner.
 * @author Gael Lalire
 */
public abstract class ArchiveScannerFactory implements IArchiveScannerFactory {

    /**
     * The RI class name.
     */
    private static final String REFERENCE_IMPLEMENTATION_CLASS_NAME = "org.ow2.util.scan.impl.ArchiveScannerFactoryImpl";

    /**
     * The current instance by tread.
     */
    private static final ThreadLocal<ArchiveScannerFactory> INSTANCE =
        new InheritableThreadLocal<ArchiveScannerFactory>();

    /**
     * @return the latest set ArchiveScannerFactory in current thread or the RI
     * @throws ScanException if no ArchiveScannerFactory set and the RI can not
     *         be accessed with current thread classloader
     */
    public static ArchiveScannerFactory getInstance() throws ScanException {
        ArchiveScannerFactory archiveScannerFactory = INSTANCE.get();
        if (archiveScannerFactory == null) {
            ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
            try {
                Class<? extends ArchiveScannerFactory> clazz = classLoader.loadClass(REFERENCE_IMPLEMENTATION_CLASS_NAME)
                        .asSubclass(ArchiveScannerFactory.class);
                archiveScannerFactory = clazz.newInstance();
                INSTANCE.set(archiveScannerFactory);
            } catch (Exception e) {
                throw new ScanException("Unable to create RI", e);
            }
        }
        return archiveScannerFactory;
    }

    /**
     * Set the ArchiveScannerFactory to use in current thread.
     * @param archiveScannerFactory the archiveScannerFactory
     */
    public static void setInstance(final ArchiveScannerFactory archiveScannerFactory) {
        INSTANCE.set(archiveScannerFactory);
    }

    /**
     * Construct an archive scanner with an archive.
     * @param archive the archive
     * @return a new archive scanner
     */
    public abstract ArchiveScanner createArchiveScanner(IArchive archive);

    /**
     * Construct an archive scanner with an archive.
     * @param archiveHelper archive helper
     * @return a new archive scanner
     */
    public abstract ArchiveScanner createArchiveScanner(IArchiveHelper archiveHelper);

}
